#include <stdio.h>
#include <windows.h>
#include <setupapi.h>
#include <devguid.h>
#include <regstr.h>
#include <tchar.h>

#include "resource.h"

static HINSTANCE _hInst=NULL;
static HWND _hDlg=NULL;
static TCHAR _szVIDPID[]=_T("Vid_04d8&Pid_fb56");

static BOOL HIDOpen(PTCHAR pszDevSearchID,PTCHAR pszDevID,PHANDLE phWrite,PHANDLE phRead)
{
	GUID InterfaceClassGuid = {0x4d1e55b2, 0xf16f, 0x11cf, 0x88, 0xcb, 0x00, 0x11, 0x11, 0x00, 0x00, 0x30}; 
	HDEVINFO hDevInfo;
	SP_DEVINFO_DATA DeviceInfoData;
	DWORD i;
	PTCHAR pszDevSearchIDLower=_tcsdup(pszDevSearchID);

	_tcslwr(pszDevSearchIDLower);

	// Create a HDEVINFO with all present devices.
	hDevInfo = SetupDiGetClassDevs
	(
		&InterfaceClassGuid,
		0, // Enumerator
		0,
		DIGCF_PRESENT | DIGCF_DEVICEINTERFACE 
	);
   
	if (hDevInfo == INVALID_HANDLE_VALUE)
	{
		// Insert error handling here.
		return FALSE;
	}
   
	// Enumerate through all devices in Set.
    DeviceInfoData.cbSize = sizeof(SP_DEVINFO_DATA);
	for (i=0;SetupDiEnumDeviceInfo(hDevInfo,i,&DeviceInfoData);i++)
	{
		DWORD DataT;
		LPTSTR buffer = NULL;
		DWORD buffersize = 0;

		while
		(
			!SetupDiGetDeviceRegistryProperty
			(
				hDevInfo,
				&DeviceInfoData,
				SPDRP_HARDWAREID,
				&DataT,
				(PBYTE)buffer,
				buffersize,
				&buffersize
			)
		)
		{
			if (GetLastError() == ERROR_INSUFFICIENT_BUFFER)
			{
				// Change the buffer size.
				if (buffer) 
				{
					LocalFree(buffer);
				}
				// Double the size to avoid problems on 
				// W2k MBCS systems per KB 888609. 
				buffer=LocalAlloc(LPTR,buffersize * 2);
			}
			else
			{
				// Insert error handling here.
				break;
			}
		}

		_tcslwr(buffer);
       
		if (_tcsstr(buffer,pszDevSearchIDLower)!=NULL)
		{
			// We have found the device, now lets try to open a handles to it

			SP_DEVICE_INTERFACE_DATA dids;

			dids.cbSize = sizeof(SP_DEVICE_INTERFACE_DATA);
			if(!SetupDiEnumDeviceInterfaces(hDevInfo, NULL, &InterfaceClassGuid, i, &dids))
			{
				break;
			}

			{
				SP_DEVICE_INTERFACE_DETAIL_DATA didds;
				PSP_DEVICE_INTERFACE_DETAIL_DATA pdidds=&didds;
				DWORD cb;

				pdidds->cbSize=sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);
				SetupDiGetDeviceInterfaceDetail(hDevInfo,&dids,NULL,0,&cb,NULL);	
				if ((pdidds=malloc(cb))==NULL)
				{
					break; // uh-oh
				}
				pdidds->cbSize=sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);
				SetupDiGetDeviceInterfaceDetail(hDevInfo,&dids,pdidds,cb,NULL,NULL);

				*phWrite=CreateFile(pdidds->DevicePath,GENERIC_WRITE,FILE_SHARE_READ | FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,0);
				if (GetLastError()!=ERROR_SUCCESS)
				{
					break;
				}
				*phRead=CreateFile(pdidds->DevicePath,GENERIC_READ,FILE_SHARE_READ | FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,0);
				if (GetLastError()!=ERROR_SUCCESS)
				{
					break;
				}
			}
			SetupDiDestroyDeviceInfoList(hDevInfo);
			free(pszDevSearchIDLower);
			_tcscpy(pszDevID,buffer);
			return TRUE;
		}
       
		if (buffer)
		{
			LocalFree(buffer);
		}
	}
	if ( GetLastError()!=NO_ERROR && GetLastError()!=ERROR_NO_MORE_ITEMS )
	{
	   return FALSE;
	}
	SetupDiDestroyDeviceInfoList(hDevInfo);
	free(pszDevSearchIDLower);
	return FALSE;
}

static void HIDClose(HANDLE hWrite,HANDLE hRead)
{
	CloseHandle(hWrite);
	CloseHandle(hRead);
}

static unsigned char HIDSetFreq(HANDLE hWrite,HANDLE hRead,int nFreq)
{
	DWORD dwBytesWritten=0;
	DWORD dwBytesRead=0;
	unsigned char au8BufOut[65]; // endpoint size + 1
	unsigned char au8BufIn[65]; // endpoint size + 1

	au8BufOut[0]=0; // First byte is report ID. Ignored by HID Class firmware as only config'd for one report
	au8BufOut[1]=100;
	au8BufOut[2]=(unsigned char)nFreq;
	au8BufOut[3]=(unsigned char)(nFreq>>8);
	au8BufOut[4]=(unsigned char)(nFreq>>16);
	WriteFile(hWrite,au8BufOut,65,&dwBytesWritten,0);
	ReadFile(hRead,au8BufIn,65,&dwBytesRead,0);
	return au8BufIn[2];
}

static unsigned char HIDResetFCD(HANDLE hWrite,HANDLE hRead)
{
	DWORD dwBytesWritten=0;
	DWORD dwBytesRead=0;
	unsigned char au8BufOut[65]; // endpoint size + 1
	unsigned char au8BufIn[65]; // endpoint size + 1

	au8BufOut[0]=0; // First byte is report ID. Ignored by HID Class firmware as only config'd for one report
	au8BufOut[1]=255;
	WriteFile(hWrite,au8BufOut,65,&dwBytesWritten,0);
	ReadFile(hRead,au8BufIn,65,&dwBytesRead,0);
	return au8BufIn[2];
}

/*
int main(int nArgs, char *apszArgs[])
{
	TCHAR sz[250];
	HANDLE hWrite=INVALID_HANDLE_VALUE;
	HANDLE hRead=INVALID_HANDLE_VALUE;
	int nFreq;
	
	if (nArgs!=2)
	{
		printf("Frequency expected\n");
		return -1;
	}

	nFreq=atoi(apszArgs[1]);
	if (nFreq<50000 || nFreq>2500000)
	{
		printf("Frequency expected to be between 50000 and 2500000\n");
		return -1;
	}

	if (OpenHIDDevice("Vid_04d8&Pid_FB56",sz,&hWrite,&hRead))
	{
		printf("Found: %s\n",sz);
		printf("%d\n",SetFreq(hWrite,hRead,nFreq));
		CloseHandle(hWrite);
		hWrite=NULL;
		CloseHandle(hRead);
		hRead=NULL;
	}
	return 0;
}
*/

static void Message(PTCHAR psz)
{
	SendMessage(GetDlgItem(_hDlg,IDC_EDITMESSAGE),EM_REPLACESEL,(WPARAM)FALSE,(LPARAM)psz);
}

static void MessageClear(void)
{
	if (_hDlg!=NULL)
	{
		SetDlgItemText(_hDlg,IDC_EDITMESSAGE,_T(""));
	}
}

static void SetFreq(int nFreq)
{
	TCHAR sz[200];
	TCHAR szMsg[400];
	BOOL b;
	HANDLE hWrite=INVALID_HANDLE_VALUE;
	HANDLE hRead=INVALID_HANDLE_VALUE;

	MessageClear();
	b=HIDOpen(_szVIDPID,sz,&hWrite,&hRead);
	_stprintf(szMsg,"HIDOpen of %s returns %d",_szVIDPID,b);
	Message(szMsg);
	if (b)
	{
		unsigned int u8;

		Message(_T("\r\nSuccess, FCD found."));
		u8=HIDSetFreq(hWrite,hRead,nFreq);
		_stprintf(szMsg,"\r\nHIDSetFreq of %d returns %d",nFreq,u8);
		Message(szMsg);
		if (u8==1)
		{
			Message(_T("\r\nSuccess, FCD frequency set."));
		}
		else
		{
			Message(_T("\r\nError, FCD frequency NOT set."));
		}
	}
	else
	{
		Message(_T("\r\nError, FCD not found: is FCD correctly installed?"));
	}
	HIDClose(hWrite,hRead);
}

static ResetFCD(void)
{
	TCHAR sz[200];
	TCHAR szMsg[400];
	BOOL b;
	HANDLE hWrite=INVALID_HANDLE_VALUE;
	HANDLE hRead=INVALID_HANDLE_VALUE;

	MessageClear();
	b=HIDOpen(_szVIDPID,sz,&hWrite,&hRead);
	_stprintf(szMsg,"HIDOpen of %s returns %d",_szVIDPID,b);
	Message(szMsg);
	if (b)
	{
		unsigned int u8;

		Message(_T("\r\nSuccess, FCD found."));
		u8=HIDResetFCD(hWrite,hRead);
		_stprintf(szMsg,"\r\nHIDReset returns %d",u8);
		Message(szMsg);
		if (u8==1)
		{
			Message(_T("\r\nSuccess, FCD reset to bootloader."));
		}
		else
		{
			Message(_T("\r\nError, FCD NOT reset."));
		}
	}
	else
	{
		Message(_T("\r\nError, FCD not found: is FCD correctly installed?"));
	}
	HIDClose(hWrite,hRead);
}

static LRESULT CALLBACK MainDlg(HWND hDlg,UINT message,WPARAM wParam,LPARAM lParam)
{
	static HFONT hFont=NULL;
	static HBRUSH hBrush;
	static const COLORREF crEditFreqBk=RGB(0,0,0);
	static const COLORREF crEditFreq=RGB(0,255,0);

	switch (message) 
	{
		case WM_INITDIALOG:
			_hDlg=hDlg;
			hBrush = CreateSolidBrush(crEditFreqBk);
		  hFont=CreateFont (48, 0, 0, 0, FW_DONTCARE, FALSE, FALSE, FALSE, ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_SWISS, _T("Arial"));
			SendDlgItemMessage(hDlg,IDC_EDITFREQ,WM_SETFONT,(WPARAM)hFont,TRUE);
			SendDlgItemMessage(hDlg,IDC_STATICKHZ,WM_SETFONT,(WPARAM)hFont,TRUE);
			break;
		case WM_DESTROY:
			DeleteObject(hFont);
			hFont=INVALID_HANDLE_VALUE;
			DeleteObject(hBrush);
			hBrush=INVALID_HANDLE_VALUE;
			_hDlg=NULL;
      return TRUE;
		case WM_TIMER:
			break;
		case WM_CTLCOLOREDIT:
			if ((HWND)lParam==GetDlgItem(hDlg,IDC_EDITFREQ))
			{
				SetTextColor((HDC)wParam,crEditFreq);
				SetBkColor((HDC)wParam,crEditFreqBk);
				return (LRESULT)hBrush;
			}
			break;
		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case IDOK:
				case IDCANCEL:
				case ID_EXIT:
					EndDialog(hDlg, TRUE);
					break;
				case ID_SETFREQ:
					{
						int n;
						TCHAR sz[30];

						GetDlgItemText(hDlg,IDC_EDITFREQ,sz,sizeof(sz)-1);
						n=atoi(sz);

						if (n<50000 || n>2500000)
						{
							MessageBox(hDlg,_T("Frequency must be between 50000 and 2500000"),_T("Error"),MB_OK | MB_ICONEXCLAMATION);
						}
						else
						{
							SetFreq(n);
						}
					}
					break;
				case ID_RESETFCD:
					ResetFCD();
					break;
				default:
					break;
			}
      return TRUE;

		default:
			break;
	}
  return FALSE;
}

int APIENTRY WinMain(HINSTANCE hInst,HINSTANCE hPrevInst,LPSTR lpCmdLine,int nCmdShow)
{
	_hInst=hInst;
	DialogBox(hInst,MAKEINTRESOURCE(IDD_MAIN),NULL,MainDlg);
  return 0;
}
